/*

# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is Mozilla Communicator client code, released
# March 31, 1998.
#
# The Initial Developer of the Original Code is
# Netscape Communications Corporation.
# Portions created by the Initial Developer are Copyright (C) 1998-1999
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****

*/

/* ===== searchDialog.css ===============================================
  == Styles for the Mail Search dialog.
  ======================================================================= */

@import url("chrome://messenger/skin/");
@import url("chrome://messenger/skin/tagColors.css");

@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");

window {
        padding-top: 8px;
}

window > vbox {
	padding: 0px 8px;
}

window > vbox > vbox {
	font: menu;
}
/* ::::: box sizes ::::: */

#searchTermTree {
  min-height: 50px;
}

#searchTermListBox {
  height: 100px;
}

#searchResultListBox {
  margin: 5px;
  height: 100px;
  font: message-box;
  border-right: 1px solid #C3C3C3;
  border-bottom: 1px solid #DDDDDD;
  border-left: 1px solid #C3C3C3;
}

.small-button {
  min-width: 3em;
  padding: 0px;
  margin: 0px 1px;
}

listbox {
  padding-top: 2px;
}

menulist {
  width: 12em;
}

menulist > menupopup > menuitem {
  padding-right: 2px;
}

textbox {
  margin: 1px 4px;
}

/* ::::: thread decoration ::::: */

treechildren::-moz-tree-cell-text(read) {
  font-weight: normal;
}

treechildren::-moz-tree-cell-text(unread) {
  font-weight: bold;
}

/* on a collapsed thread, if the top level message is read, but the thread has
 * unread children, underline the text.  4.x mac did this, very slick 
 */
treechildren::-moz-tree-cell-text(container, closed, hasUnread, read) {
  text-decoration: underline;
}

/* ::::: message column icons ::::: */

/* ..... thread column ..... */

treecol.threadColumnHeader {
  list-style-image: url("chrome://messenger/skin/icons/unthreadcol.png");
}

treecol.threadColumnHeader[sortDirection="ascending"]  {
  list-style-image: url("chrome://messenger/skin/icons/threadcol.png");
}

treecol.threadColumnHeader[sortDirection="descending"] {
  list-style-image: url("chrome://messenger/skin/icons/threadcol.png");
}

treechildren::-moz-tree-image(threadCol, container) {
  list-style-image: url("chrome://messenger/skin/icons/thread-closed.png");
}

treechildren::-moz-tree-image(threadCol, container, open) {
  list-style-image: url("chrome://messenger/skin/icons/thread-open.png");
}

treechildren::-moz-tree-image(threadCol, container, hasUnread) {
  list-style-image: url("chrome://messenger/skin/icons/thread-new-closed.png");
}

treechildren::-moz-tree-image(threadCol, container, hasUnread, open) {
  list-style-image: url("chrome://messenger/skin/icons/thread-new-open.png");
}

/* ..... read column ..... */

treecol.readColumnHeader {
  list-style-image: url("chrome://messenger/skin/icons/readcol.png");
}

treechildren::-moz-tree-image(unreadButtonColHeader) {
  list-style-image: url("chrome://messenger/skin/icons/readmail.png");
}

treechildren::-moz-tree-image(unreadButtonColHeader, unread) {
  list-style-image: url("chrome://messenger/skin/icons/unreadmail.png");
}

/* ..... flag column ..... */

treecol.flagColumnHeader {
  list-style-image: url("chrome://messenger/skin/icons/flagcol.png");
}

treechildren::-moz-tree-image(flaggedCol) {
  list-style-image: url("chrome://messenger/skin/icons/readmail.png");
}

treechildren::-moz-tree-image(flaggedCol, flagged) {
  list-style-image: url("chrome://messenger/skin/icons/flaggedmail.png");
}

/* ..... junkStatus column ..... */

treecol.junkStatusHeader {
  list-style-image: url("chrome://messenger/skin/icons/folder-junk.png");
  padding: 0px 4px 0px 0px;
}

/* "unknown" now looks like "not junk".  see bug #182386 */
treechildren::-moz-tree-image(junkStatusCol) {
  list-style-image: url("chrome://messenger/skin/icons/readmail.png");
  padding-left: 0px;
  padding-right: 4px;
}

treechildren::-moz-tree-image(junkStatusCol, junk) {
  list-style-image: url("chrome://messenger/skin/icons/folder-junk.png");
  margin-right: -1px;
}

treechildren::-moz-tree-image(junkStatusCol, notjunk) {
  list-style-image: url("chrome://messenger/skin/icons/readmail.png");
  padding-left: 0px;
  padding-right: 4px;
}

/* ..... subject column ..... */

treechildren::-moz-tree-image(subjectCol) {
  margin-right: 2px;
  list-style-image: url("chrome://messenger/skin/icons/message-mail.png");
}

treechildren::-moz-tree-image(subjectCol, new) {
  list-style-image: url("chrome://messenger/skin/icons/message-mail-new.png");
}

treechildren::-moz-tree-image(subjectCol, attach) {
  list-style-image: url("chrome://messenger/skin/icons/message-mail-attach.png");
}

treechildren::-moz-tree-image(subjectCol, imapdeleted) {
  list-style-image: url("chrome://messenger/skin/icons/message-mail-imapdelete.png");
}

treechildren::-moz-tree-image(subjectCol, news) {
  list-style-image: url("chrome://messenger/skin/icons/message-news.png");
}

treechildren::-moz-tree-image(subjectCol, news, attach) {
  list-style-image: url("chrome://messenger/skin/icons/message-news-attach.png");
}

treechildren::-moz-tree-image(subjectCol, news, attach, offline) {
  list-style-image: url("chrome://messenger/skin/icons/message-news-attach-offl.png");
}

treechildren::-moz-tree-image(subjectCol, news, new) {
  list-style-image: url("chrome://messenger/skin/icons/message-news-new.png");
}

treechildren::-moz-tree-image(subjectCol, news, new, attach) {
  list-style-image: url("chrome://messenger/skin/icons/message-news-new-attach.png");
}

/* ..... new thread icons for watch and ignore ..... */

treechildren::-moz-tree-image(news, threadCol, watch) {
  list-style-image: url("chrome://messenger/skin/icons/thread-closed-eye.png");
}

treechildren::-moz-tree-image(news, threadCol, ignore) {
  list-style-image: url("chrome://messenger/skin/icons/thread-closed-kill.png");
}

treechildren::-moz-tree-image(news, threadCol, container, hasUnread, watch) {
  list-style-image: url("chrome://messenger/skin/icons/thread-new-closed-eye.png");
}

treechildren::-moz-tree-image(news, threadCol, container, hasUnread, ignore) {
  list-style-image: url("chrome://messenger/skin/icons/thread-new-closed-kill.png");
}

treechildren::-moz-tree-image(news, threadCol, container, hasUnread, watch, open) {
  list-style-image: url("chrome://messenger/skin/icons/thread-new-open-eye.png");
}

treechildren::-moz-tree-image(news, threadCol, container, hasUnread, ignore, open) {
  list-style-image: url("chrome://messenger/skin/icons/thread-new-open-kill.png");
}

treechildren::-moz-tree-image(news, threadCol, container, watch, open) {
  list-style-image: url("chrome://messenger/skin/icons/thread-open-eye.png");
}

treechildren::-moz-tree-image(news, threadCol, container, ignore, open) {
  list-style-image: url("chrome://messenger/skin/icons/thread-open-kill.png");
}

#sizeCol,
#unreadCol,
#totalCol {
  text-align: right;
}

#status-bar {
	font: message-box;
	margin-top: 8px;
}
